; This file is part of Magical Fairy Force.

; Magical Fairy Force is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Magical Fairy Force is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Magical Fairy Force.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2020 Average Software.

; Common joystick/trakball handling routines

; Controller type constants.
controller_stick    equ 0
controller_ball     equ 1
controller_cpu      equ 2

; Storage for the controller types.
p0_controller_type  equ $2f0
p1_controller_type  equ $2f1

; Trak-ball rest values.
p0_ball_rest_h      equ $2f2
p0_ball_rest_v      equ $2f3
p1_ball_rest_h      equ $2f4
p1_ball_rest_v      equ $2f5

; Input values for each player.  Values are
; interpreted from both input sources and
; centered around 0.  Negative values are
; left/up.
p0_h_input          equ $2f6
p0_v_input          equ $2f7
p1_h_input          equ $2f8
p1_v_input          equ $2f9
p0_trig1            equ $2fa
p0_trig2            equ $2fb
p1_trig1            equ $2fc
p1_trig2            equ $2fd

; Active controller for trigger 2 reads.
active_controller   equ $2fe

; CPU constants.
; State constants.
cpu_seek_pos        equ 0
cpu_move_to_pos     equ 1

; Detect the types of controllers connected.
detect_controllers subroutine
    mva     #0,CONSOL               ; Disable pot scanning.
    ldy     #2
    jsr     wait_for_y_frames       ; Wait for the shadows to update.
    lda     sPOT0                   ; Check the first port.
    cmp     #$e4                    ; Joystick reads $e4 when disabled.
    bne     .p0_ball
    lda     #controller_stick       ; Joystick, or nothing perhaps.
    beq     .store_p0               ; Always true.
.p0_ball
    sta     p0_ball_rest_h          ; This is the horizontal rest value.
    lda     sPOT1                   ; Get the vertical rest value.
    sta     p0_ball_rest_v
    lda     #controller_ball
.store_p0
    sta     p0_controller_type
    lda     sPOT2                   ; Repeat for port 2.
    cmp     #$e4
    bne     .p1_ball
    lda     #controller_stick
    beq     .store_p1               ; Always true.
.p1_ball
    sta     p1_ball_rest_h
    lda     sPOT3
    sta     p1_ball_rest_v
    lda     #controller_ball
.store_p1
    sta     p1_controller_type
    mva     #4,CONSOL               ; Re-enable pot scanning.
    sta     active_controller
    ldy     #2
    jmp     wait_for_y_frames       ; Allow the shadows to update.

; Read the input devices and convert to
; generalized input values.
read_inputs subroutine
    lda     TRIG0                   ; Assume human players, the CPU
    sta     p0_trig1                ; will clobber these later.
    lda     TRIG1
    sta     p1_trig1
    lda     #0                      ; Assume trigger 2 is not pressed.
    sta     p0_trig2
    sta     p1_trig2
    lda     SKSTAT                  ; Check for trigger 2.
    and     #8
    bne     .no_top_trigger
    lda     active_controller       ; See who pressed it.
    and     #1
    beq     .p0_top_trigger
    sta     p1_trig2
    bne     .no_top_trigger         ; Always true.
.p0_top_trigger
    lda     #1
    sta     p0_trig2
.no_top_trigger
    lda     active_controller       ; Flip the active controller.
    eor     #1
    sta     active_controller
    sta     CONSOL
    lda     p0_controller_type
    beq     .p0_stick_input
    mva     p0_ball_rest_h,$ff
    lda     sPOT0
    jsr     interpret_trak_ball
    sta     p0_h_input
    mva     p0_ball_rest_v,$ff
    lda     sPOT1
    jsr     interpret_trak_ball
    sta     p0_v_input
    jmp     .read_p1
.p0_stick_input
    lda     sPOT0
    jsr     interpret_stick
    sta     p0_h_input
    lda     sPOT1
    jsr     interpret_stick
    sta     p0_v_input
.read_p1
    lda     p1_controller_type
    beq     .p1_stick_input
    cmp     #controller_cpu
    beq     .p1_cpu_input
    mva     p1_ball_rest_h,$ff
    lda     sPOT2
    jsr     interpret_trak_ball
    sta     p1_h_input
    mva     p1_ball_rest_v,$ff
    lda     sPOT3
    jsr     interpret_trak_ball
    sta     p1_v_input
    rts
.p1_stick_input
    lda     sPOT2
    jsr     interpret_stick
    sta     p1_h_input
    lda     sPOT3
    jsr     interpret_stick
    sta     p1_v_input
    rts
.p1_cpu_input
    lda     RANDOM                  ; Check for an attack.
    cmp     cpu_attack_odds
    bcs     .no_cpu_attack
    lda     cpu_big_attack          ; Saving for big attack?
    beq     .small_attack
    lda     p1_fairy_power          ; Check saved power.
    cmp     #max_fairy_power
    bne     .no_cpu_attack          ; Keep waiting.
.small_attack
    lda     RANDOM                  ; Check for big attack.
    cmp     cpu_attack_odds
    bcs     .no_big_attack
    lda     #1
    sta     cpu_big_attack
.no_big_attack
    lda     #1
    .byte   $2c
.no_cpu_attack
    lda     #0                      ; No second trigger.
    sta     p1_trig2
    lda     #1                      ; Assume no normal shot.
    sta     p1_trig1
    lda     cpu_state               ; Check the CPU state.
    beq     .seek_new_pos           ; Find a new position.
    jmp     .move_to_pos            ; Check movement to the target.
.seek_new_pos
    lda     p0_attack_active        ; If under attack, randomly choose to
    beq     .no_attack_active       ; avoid the attack or pursue a star.
    lda     RANDOM
    cmp     cpu_star_seek_odds
    bcc     .seek_star
    lda     #-1                     ; Clear the cloud and star targets.
    sta     cpu_cloud_index
    sta     cpu_star_index
    jmp     (cpu_attack_routine)
.no_attack_active
    lda     RANDOM                  ; 50/50 cloud/star pursuit.
    bmi     .seek_cloud
.seek_star
    ldy     #1                      ; Star index.
.star_search_loop
    lda     p1_star_0_index,y
    bpl     .star_found
    dey
    bpl     .star_search_loop
.star_found
    sty     cpu_star_index          ; Save the index even if not found.
    cpy     #-1                     ; Was a star found?
    beq     .seek_cloud             ; If not, look for a cloud instead.
    lda     #39
    sec
    sbc     p1_star_0_index,y
    jsr     star_index_to_hpos
    sta     cpu_xdest               ; Leave the vertical alone.
    cmp     p1_real_hpos            ; Check direction of movement.
    bcs     .star_move_right
    lda     cpu_left_speed
    .byte   $2c
.star_move_right
    lda     cpu_right_speed
    sta     cpu_hspeed
    jsr     cpu_center_backoff
    jmp     .target_ready
.seek_cloud
    ldy     #7                      ; Find the lowest cloud.
    lda     #$ff
    sta     $ff                     ; Current lowest vpos.
    sta     $fe                     ; Current lowest index.
    sta     cpu_star_index          ; Not seeking a star.
.lowest_cloud_loop
    lda     p1_cloud_hpos,y         ; Hpos is -1 when inactive.
    cmp     #-1
    beq     .next_cloud
    lda     p1_cloud_vpos,y         ; Compare against the lowest vpos.
    cmp     $ff
    bcs     .next_cloud
    sta     $ff                     ; This is lower, update the vpos.
    sty     $fe                     ; Update the index.
.next_cloud
    dey
    bpl     .lowest_cloud_loop
    ldy     $fe                     ; Was a cloud found?
    sty     cpu_cloud_index         ; Save the result either way.
    bmi     .no_cloud_found
    lda     p1_cloud_type,y         ; Take action based on the cloud type.
    beq     .react_to_cloud
    cmp     #spawn_fairy_dust
    beq     .react_to_fairy_dust
    lda     #p1_top_bound           ; Head for the top of the screen.
    bne     .store_vmove            ; Always true.
.react_to_cloud
    lda     #p1_top_bound           ; Head for the top of the screen.
    bne     .store_vmove            ; Always true.
.react_to_fairy_dust
    lda     p1_cloud_vpos,y         ; Lead the dust a bit.
    sec
    sbc     #10
.store_vmove
    sta     cpu_ydest
    lda     p1_cloud_hpos,y         ; Hpos is the same regardless of type.
    sta     cpu_xdest
    cmp     p1_real_hpos
    bcs     .move_right
    lda     cpu_left_speed
    .byte   $2c
.move_right
    lda     cpu_right_speed
    sta     cpu_hspeed
    lda     cpu_ydest
    cmp     p1_vpos
    bcs     .move_down
    lda     cpu_up_speed
    .byte   $2c
.move_down
    lda     cpu_down_speed
    sta     cpu_vspeed
    jmp     .target_ready
.no_cloud_found
    lda     p1_real_hpos ; TEMP
    sta     cpu_xdest ; TEMP
    lda     p1_vpos ; TEMP
    sta     cpu_ydest ; TEMP
    lda     #0 ; TEMP
    sta     cpu_hspeed ; TEMP
    sta     cpu_vspeed; TEMP
.target_ready
    inc     cpu_state
.move_to_pos
    lda     cpu_star_avoid
    beq     .skip_star_avoid
    ldy     #p1w0_shot              ; Check for star shot threats.
    jsr     cpu_evaluate_star_shot
    ldy     #p1w1_shot
    jsr     cpu_evaluate_star_shot
.skip_star_avoid
    ldy     cpu_cloud_index         ; See if the cloud being reacted to
    bmi     .no_target_cloud        ; is still active.
    lda     p1_cloud_hpos,y
    cmp     #-1
    bne     .no_target_cloud
    jmp     .cancel_cloud
.no_target_cloud
    ldy     cpu_star_index          ; See if the star being reacted to
    bmi     .no_target_star         ; is still active.
    lda     p1_star_0_index,y
    bpl     .no_target_star
    jmp     .cancel_star
.no_target_star
    lda     p1_real_hpos
    ldx     cpu_hspeed              ; Check movement direction.
    bmi     .check_left
    cmp     cpu_xdest               ; See if the destination has been
    bcs     .clamp_xdest_right      ; reached or passed.
    cmp     #right_bound            ; Check the right screen edge.
    bcs     .clamp_xdest_right
    bcc     .hmove_done             ; Always true.
.clamp_xdest_right
    lda     cpu_xdest
    ldy     #right_bound
    jsr     get_min_value
    jmp     .xdest_reached
.check_left
    cmp     cpu_xdest
    beq     .xdest_reached
    bcc     .clamp_xdest_left
    cmp     #left_bound             ; Check the left screen edge.
    beq     .xdest_reached
    bcc     .clamp_xdest_left
    bcs     .hmove_done             ; Always true.
.clamp_xdest_left
    lda     cpu_xdest
    ldy     #left_bound
    jsr     get_max_value
.xdest_reached
    sta     cpu_xdest               ; Hard set the target in case
    sta     p1_real_hpos            ; we passed it.
    sec
    sbc     #4
    sta     p1_real_wing_hpos
    ldx     #0                      ; Stop horizontal movement.
    stx     cpu_hspeed
.hmove_done
    stx     p1_h_input              ; Store the move.
    lda     p1_vpos                 ; Repeat for the vertical.
    ldx     cpu_vspeed
    bmi     .check_up
    cmp     cpu_ydest
    bcs     .clamp_ydest_bottom
    cmp     #p1_bottom_move_bound
    bcs     .clamp_ydest_bottom
    bcc     .vmove_done             ; Always true.
.clamp_ydest_bottom
    lda     cpu_ydest
    ldy     #p1_bottom_move_bound
    jsr     get_min_value
    jmp     .ydest_reached
.check_up
    cmp     cpu_ydest
    beq     .ydest_reached
    bcc     .clamp_ydest_top
    cmp     #p1_top_bound
    beq     .ydest_reached
    bcc     .clamp_ydest_top
    bcs     .vmove_done             ; Always true.
.clamp_ydest_top
    lda     cpu_ydest
    ldy     #p1_top_bound
    jsr     get_max_value
.ydest_reached
    cmp     p1_vpos                 ; Adjust if necessary.
    beq     .no_vadjust
    pha                             ; Save the destination.
    bcc     .vadjust_up
    sbc     p1_vpos
    tay                             ; Rows to move.
    lda     p1_vpos
    jsr     move_player_down
    jmp     .vadjust_done
.vadjust_up
    sta     $ff
    lda     p1_vpos
    sec
    sbc     $ff
    tay                             ; Rows to move.
    lda     p1_vpos
    jsr     move_player_up
.vadjust_done
    pla                             ; Get the destination back.
.no_vadjust
    sta     cpu_ydest
    sta     p1_vpos
    ldx     #0
    stx     cpu_vspeed
.vmove_done
    stx     p1_v_input
    lda     cpu_hspeed              ; If both speeds are 0, find a new
    bne     .move_active            ; destination to go to.
    lda     cpu_vspeed
    bne     .move_active
    lda     #0                      ; Fire a shot.
    sta     p1_trig1
    beq     .finish_seek            ; Always true.
.cancel_cloud
    lda     #-1
    sta     cpu_cloud_index         ; Harmless fall-through.
.cancel_star
    lda     #-1
    sta     cpu_star_index
.finish_seek
    dec     cpu_state
    lda     #1
    sta     cpu_star_avoid
.move_active
    rts

; Back off the center of the screen if too close.
cpu_center_backoff subroutine
    lda     #p1_bottom_move_bound - 20
    cmp     p1_vpos
    bcc     .backoff
    rts
.backoff
    lda     #p1_top_bound
    sta     cpu_ydest
    lda     cpu_up_speed
    sta     cpu_vspeed
    rts

; Check for incoming star shot hazards.  Expects the
; shot data base address in the y register.
cpu_evaluate_star_shot subroutine
    lda     shot_hpos,y             ; Check for activity.
    beq     .inactive
    sec                             ; Check for x intersection.
    sbc     p1_real_hpos
    sta     $fc                     ; Save the distance.
    cmp     #12
    bcc     .x_intersect
    cmp     #-4
    bcc     .no_x_intersect
.x_intersect
    lda     #1                      ; Flag it.
    .byte   $2c
.no_x_intersect
    lda     #0                      ; Flag it.
    sta     $ff                     ; Store the result.
    lda     shot_vpos,y             ; Check for y intersection.
    sec
    sbc     p1_vpos
    sta     $fd                     ; Save the distance.
    cmp     #player_sprite_rows + 4
    bcc     .y_intersect
    cmp     #-4
    bcc     .no_y_intersect
.y_intersect
    lda     #1                      ; Flag it.
    .byte   $2c
.no_y_intersect
    lda     #0                      ; Flag it.
    sta     $fe                     ; Store the result.
    bne     .check_h_distance
    lda     $ff
    bne     .inactive               ; No intersection, skip.
    lda     $fd                     ; Make sure the shot is close.
    cmp     #-8
    bcc     .inactive               ; Not close enough.
    cmp     #player_sprite_rows + 8
    bcs     .inactive               ; Not close enough.
.check_h_distance
    lda     $fc                     ; Make sure the shot is close.
    cmp     #-8
    bcs     .shot_on_left
    cmp     #16
    bcs     .inactive               ; Not close enough, ignore.
    lda     p1_real_hpos            ; Move left.
    sbc     #10                     ; Carry is set.
    sta     cpu_xdest
    lda     cpu_left_speed
    bne     .store_hspeed           ; Always true.
.shot_on_left
    lda     p1_real_hpos            ; Move right.
    adc     #10                     ; Carry is clear.
    sta     cpu_xdest
    lda     cpu_right_speed
.store_hspeed
    sta     cpu_hspeed
    lda     p1_vpos
    sta     cpu_ydest
    ldx     #0
    stx     cpu_vspeed
    stx     cpu_star_avoid
    dex
    stx     cpu_cloud_index
    stx     cpu_star_index
.inactive
    rts

; Convert a trak-ball value to a generalized input.
; Expects the raw value in the accumulator and the
; rest value in $ff.  Places the result in the accumulator.
interpret_trak_ball subroutine
    sec                             ; Subtract the rest value.
    sbc     $ff
    cmp     #-10
    bmi     .minus_4
    cmp     #-8
    bmi     .minus_3
    cmp     #-6
    bmi     .minus_2
    cmp     #-4
    bmi     .minus_1
    cmp     #4
    bmi     .neutral
    cmp     #6
    bmi     .plus_1
    cmp     #8
    bmi     .plus_2
    cmp     #10
    bmi     .plus_3
.plus_4
    lda     #4
    rts
.minus_4
    lda     #-4
    rts
.minus_3
    lda     #-3
    rts
.minus_2
    lda     #-2
    rts
.minus_1
    lda     #-1
    rts
.neutral
    lda     #0
    rts
.plus_1
    lda     #1
    rts
.plus_2
    lda     #2
    rts
.plus_3
    lda     #3
    rts

; Convert a joystick value to a generalized input.
; Called as above.   Not a new subroutine, labels
; are shared with the above routine.
interpret_stick
    cmp     #25
    bcc     .minus_2
    cmp     #60
    bcc     .minus_1
    cmp     #160
    bcc     .neutral
    cmp     #195
    bcc     .plus_1
    bcs     .plus_2                 ; Always true.

