; This file is part of Magical Fairy Force.

; Magical Fairy Force is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Magical Fairy Force is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Magical Fairy Force.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2020 Average Software.

; General utility routines

; Place the sound data on page 7.
sound_pointer_1     equ     $700
sound_timer_1       equ     $702
sound_control_1     equ     $703
sound_pointer_2     equ     $704
sound_timer_2       equ     $706
sound_control_2     equ     $707
sound_pointer_3     equ     $708
sound_timer_3       equ     $70a
sound_control_3     equ     $70b
sound_pointer_4     equ     $70c
sound_timer_4       equ     $70e
sound_control_4     equ     $70f

; Busy-waits until vertical blank.
wait_for_vb subroutine
    lda     RCLKLO                  ; Read the low byte of the clock.
.loop
    cmp     RCLKLO                  ; Loop until it changes.
    beq     .loop
    rts

; Pause for the number of frames in y.
wait_for_y_frames subroutine
    jsr     wait_for_vb
    dey
    bne     wait_for_y_frames
    rts

; Copies the memory located at $fc, $fd to $fe, $ff.
; Number of bytes to copy should be in y.
copy_memory subroutine
    dey
    cpy     #$ff
    beq     .done                   ; Done after underflow.
    lda     ($fc),y                 ; Read a byte.
    sta     ($fe),y                 ; Store a byte.
    jmp     copy_memory             ; Do it again.
.done
    rts

; Puts the higher of the values in y and the
; accumulator into the accumulator.  Clobbers $ff.
get_max_value subroutine
    sty     $ff
    cmp     $ff
    bcs     .accumulator_wins
    tya
.accumulator_wins
    rts

; Same as get_max_value, but gets the minimum.
get_min_value subroutine
    sty     $ff
    cmp     $ff
    bcc     .accumulator_wins
    tya
.accumulator_wins
    rts

; Copy memory in reverse.  Same arguments as copy_memory.
copy_memory_reverse subroutine
    mva     #0,$fa                  ; Target index.
    sty     $fb                     ; Save the source index.
.loop
    ldy     $fb                     ; Get the source index.
    dey
    cpy     #$ff
    beq     .done                   ; Done after underflow.
    lda     ($fc),y                 ; Read a byte.
    ldy     $fa
    sta     ($fe),y                 ; Store byte.
    inc     $fa                     ; Increment the target index.
    dec     $fb                     ; Decrement the source index.
    jmp     .loop                   ; Do it again.
.done
    rts

; Move all players and missiles offscreen.
hide_all_sprites subroutine
    ldy     #7
    lda     #0
.loop
    sta     HPOSP0,y
    dey
    bpl     .loop
    rts

; Utility functions for portrait copies.
copy_portrait_update_write_index subroutine
    iny
    rts

copy_portrait_update_write_index_reversed subroutine
    dey
    rts

copy_portrait_prep_byte_reversed subroutine
    lda     $1100,y
    ldy     $f4                     ; Restore the write index.
    rts

copy_portrait_prep_byte subroutine
    tya                             ; Keep the byte as is.
    ldy     $f4                     ; Restore the write index.
    rts

; Copy player portrait data.  Source pointer should be
; in $f6-$f7, target should be in $f0-$f1.  If 0 is
; passed in the accumulator, the entire portrait is copied.
; Otherwise, the accumulator should contain the number of rows
; to skip for an in-game portrait.  In this case, only 10 rows
; will be copied.
copy_portrait subroutine
    ldy     #14                     ; Row end index.
    sty     $fa
    mwy     copy_portrait_prep_byte,$fd
    mwy     copy_portrait_update_write_index,$fb
    ldy     #0                      ; Initial write index.
    ; Fall through.

; Common portrait copy algorithm.
copy_portrait_common subroutine
.row_count      equ $f2
.block_size     equ $f3
.write_index    equ $f4
.skip_rows      equ $f8
.write_start    equ $f9
.write_end      equ $fa
    sta     .skip_rows
    cmp     #0
    beq     .full_portrait
    clc
    adc     #10                     ; 10 rows in a game portrait.
    .byte   $2c
.full_portrait
    lda     #60                     ; 60 rows in a portrait.
    sta     .row_count              ; Save it.
    sty     .write_index
    sty     .write_start
.outer_loop
    jsr     decompress_portrait_chunk
    cpx     #0                      ; Check for a negative size.
    bpl     .uncompressed_chunk
    txa                             ; Mask off the high bit.
    and     #$7f
    sta     .block_size             ; Save the block size.
    ldy     $1000                   ; Get the byte to repeat.
    jsi     ($fd)
    tax
    ldy     .write_index
.inner_loop_compressed
    lda     .skip_rows
    bne     .skip_write_compressed
    txa
    sta     ($f0),y
.skip_write_compressed
    cpy     .write_end              ; Check for row end.
    bne     .row_not_done_compressed
    dec     .row_count              ; Check the row count.
    beq     .done
    jsr     next_portrait_row
    lda     .skip_rows
    beq     .no_skip_compressed
    dec     .skip_rows
.no_skip_compressed
    ldy     .write_start            ; Reset the write index.
    jmp     .next_inner_loop_compressed
.row_not_done_compressed
    jsi     ($fb)                   ; Update the write index.
.next_inner_loop_compressed
    sty     .write_index            ; Next write index.
    dec     .block_size             ; Check for end of block.
    beq     .outer_loop
    bne     .inner_loop_compressed  ; Always true.
.uncompressed_chunk
    stx     .block_size             ; Save the block size.
    ldx     #0                      ; Read index.
    ldy     .write_index
.inner_loop
    lda     .skip_rows
    bne     .skip_write
    ldy     $1000,x                 ; Get a byte.
    jsi     ($fd)                   ; Prepare the byte.
    sta     ($f0),y                 ; Write the byte.
.skip_write
    cpy     .write_end              ; Check for row end.
    bne     .row_not_done
    dec     .row_count              ; Check the row count.
    beq     .done
    jsr     next_portrait_row
    lda     .skip_rows
    beq     .no_skip
    dec     .skip_rows
.no_skip
    ldy     .write_start            ; Reset the write index.
    jmp     .next_inner_loop
.row_not_done
    jsi     ($fb)                   ; Update the write index.
.next_inner_loop
    sty     .write_index
    dec     .block_size             ; Check for end of block.
    bne     .block_not_done
    jmp     .outer_loop
.block_not_done
    inx                             ; Next read index.
    bne     .inner_loop             ; Always true.
.done
    rts

; Copy player portrait data and mirror it horizontally.
; Expects the same arguments as above.
copy_portrait_mirrored subroutine
    ldy     #0                      ; Row end index.
    sty     $fa
    mwy     copy_portrait_prep_byte_reversed,$fd
    mwy     copy_portrait_update_write_index_reversed,$fb
    ldy     #14                     ; Initial write index.
    jmp     copy_portrait_common

; Advance the portrait write pointer.
next_portrait_row subroutine
    lda     $f8                     ; Skip rows active, do nothing.
    bne     .done
    lda     $f0
    clc
    adc     #40
    sta     $f0
    bcc     .done
    inc     $f1
.done
    rts

; Build a hash table of reversed bytes to aid in
; portrait flipping.  Uses $1100 since that's wasted
; by the PM graphics.
build_byte_reverse_table subroutine
    ldy     #0                      ; Hash every byte.
.loop
    tya                             ; The index is the byte to flip.
    rol                             ; Mirror the leftmost pixel.
    rol
    rol
    and     #$03                    ; Mask off the other pixels.
    sta     $ff                     ; Store the in-progress byte.
    tya                             ; Get the original back.
    lsr                             ; Mirror pixel 2.
    lsr
    and     #$0c                    ; Mask off the other pixels.
    ora     $ff                     ; Merge with the first pixel.
    sta     $ff                     ; Write it.
    tya                             ; Repeat for pixel 3.
    asl
    asl
    and     #$30
    ora     $ff
    sta     $ff
    tya                             ; Repeat for pixel 4.
    ror
    ror
    ror
    and     #$c0
    ora     $ff
    sta     $1100,y                 ; Save the reversed byte.
    iny                             ; Next byte.
    bne     .loop                   ; Stop on overflow.
    rts

; Decompress the next chunk of the portrait data.
; Decompressed data is placed at $1000, and the length
; of the decompressed data is returned in x.  If this
; was not a compressed range, the length is the true
; data length and all the data is at $1000.  If this
; was a compressed range, the length has bit 7 set and
; the compressed byte is at $1000.
decompress_portrait_chunk subroutine
    ldy     #0                      ; Read index.
    lda     ($f6),y                 ; Get the block byte.
    bmi     .compressed_chunk       ; High bit is a compressed section.
    tax                             ; Value is number of bytes in chunk.
.uncompressed_loop
    iny                             ; Next byte.
    lda     ($f6),y                 ; Read the data.
    sta     $0fff,y                 ; Write the data.  Read index is +1.
    dex
    bne     .uncompressed_loop
    iny                             ; Value to add to portrait pointer.
    tya
    tax                             ; Return the size.
    dex                             ; Size is one less than the read index.
    bne     .done                   ; Always true.
.compressed_chunk
    tax                             ; Set the length.
    iny                             ; Next byte.
    lda     ($f6),y                 ; Read the compressed byte.
    sta     $1000                   ; Save it at $1000.
    lda     #2                      ; Advance portrait pointer by 2 bytes.
.done
    clc                             ; Update the portrait pointer.
    adc     $f6
    sta     $f6
    bcc     .no_carry
    inc     $f7
.no_carry
    rts

; Transfer text from the address in $fc-$fd to the
; address in $fe-$ff.  Copies until a 0 is found.
; This version does not handle lowercase letters,
; but is faster as a result.
copy_text subroutine
    sec
    ldy     #0
.loop
    lda     ($fc),y
    beq     .done
    sbc     #$20
    sta     ($fe),y
    iny
    jmp     .loop
.done
    rts

; As above, but this version handles lowercase letters.
copy_text_full subroutine
    ldy     #0
.loop
    lda     ($fc),y
    beq     .done
    cmp     #97
    bcs     .lowercase
    sec
    sbc     #$20
.lowercase
    sta     ($fe),y
    iny
    jmp     .loop
.done
    rts

; Process all sounds.
process_sounds subroutine
    ; This macro processes a sound channel.
    ; Arguments are:
    ; {1} - The base address of the sound data block.
    ; {2} - The AUDC register for this channel.
    ; {3} - The AUDF register for this channel.
    mac     process_channel
    ; 1. Check the sound timer.
    ;   If 0 =>
    ;       Check the sound pointer.
    ;           If 0 =>
    ;               Do nothing, no sound is playing.
    ;           else =>
    ;               Advance the pointer and load the new sound data.
    ;   If -1 =>
    ;       This is a continuous sound, so do nothing.
    ;   else =>
    ;       Decrement the timer.
    lda     {1} + 2                 ; Load the timer.
    beq     .advance_sound          ; Timer expired, change the tone.
    cmp     #-1
    beq     .sound_done             ; -1, continuous sound, do nothing.
    dec     {1} + 2                 ; Decrement the timer.
    jmp     .sound_done
.advance_sound
    lda     {1} + 1                 ; Check the high byte of the pointer.
    beq     .sound_done             ; 0 byte, no sound is playing.
    lda     {1}                     ; Advance the pointer.
    clc
    adc     #2
    sta     {1}
    sta     $fe                     ; Put it where it can be dereferenced.
    lda     {1} + 1
    adc     #0
    sta     {1} + 1
    sta     $ff                     ; Put it where it can be dereferenced.
    ldy     #0
    lda     ($fe),y                 ; Get the next AUDF value.
    beq     .clear_sound            ; If 0, the sound has finished.
    cmp     #$ff                    ; If $ff, this is a rest.
    bne     .not_resting
    lda     #0                      ; Play nothing.
    sta     {2}
    jmp     .update_timer
.not_resting
    cmp     #$fe                    ; If $fe, this is an AUDC change.
    bne     .freq_change
    iny
    lda     ($fe),y                 ; Get the new AUDC.
    sta     {2}                     ; Write to AUDC.
    sta     {1} + 3                 ; Update the stored AUDC.
    jmp     .advance_sound          ; Move to the next value.
.freq_change
    sta     {3}                     ; Put the new value in AUDF.
    lda     {1} + 3                 ; Set AUDC.
    sta     {2}
.update_timer
    iny
    lda     ($fe),y                 ; Get the new timer value.
    sta     {1} + 2                 ; Update the timer.
    jmp     .sound_done
.clear_sound
    sta     {1} + 1                 ; Blank the high byte of the pointer.
    sta     {2}                     ; Clear AUDC
    sta     {3}                     ; Clear AUDF
.sound_done
    endm

    lda     $fe                     ; Preserve $fe and $ff, this gets
    pha                             ; called from interrupts.
    lda     $ff
    pha
    process_channel sound_pointer_1,AUDC1,AUDF1
    process_channel sound_pointer_2,AUDC2,AUDF2
    process_channel sound_pointer_3,AUDC3,AUDF3
    process_channel sound_pointer_4,AUDC4,AUDF4
    pla                             ; Restore $fe and $ff.
    sta     $ff
    pla
    sta     $fe
    rts

; This macro defines subroutines that play sounds in the
; various channels.  The sound should have its address 
; placed in $fe-$ff.
; Arguments are:
; {1} the AUDC register.
; {2} the AUDF register.
; {3} the sound pointer.
; {4} the timer location.
; {5} the control location.
    mac start_sound
    ldy     #0                      ; 0 index.
    lda     ($fe),y                 ; Load the AUDC value.
    sta     {1}                     ; Fill AUDC.
    sta     {5}                     ; Save AUDC for rests.
    iny                             ; Step forward.
    lda     ($fe),y                 ; Load the AUDF value.
    sta     {2}                     ; Fill AUDF.
    iny                             ; Step forward.
    lda     ($fe),y                 ; Get the frame count.
    sta     {4}                     ; Save it.
    inc     $fe                     ; Move the pointer to the first record.
    bne     .no_carry
    inc     $ff
.no_carry
    lda     $fe                     ; Store the record address.
    sta     {3}
    lda     $ff
    sta     {3} + 1
    rts
    endm

; Sound subroutines.  Defined by the macro above.
play_sound_1 subroutine
    start_sound AUDC1,AUDF1,sound_pointer_1,sound_timer_1,sound_control_1

play_sound_2 subroutine
    start_sound AUDC2,AUDF2,sound_pointer_2,sound_timer_2,sound_control_2

play_sound_3 subroutine
    start_sound AUDC3,AUDF3,sound_pointer_3,sound_timer_3,sound_control_3

play_sound_4 subroutine
    start_sound AUDC4,AUDF4,sound_pointer_4,sound_timer_4,sound_control_4

; This macro defines routines that halt the sound on
; a particular channel.
; Arguments are:
; {1} the AUDC register.
; {2} the AUDF register.
; {3} the sound pointer.
; {4} the timer location.
    mac stop_sound
    lda     #0                      ; Zero out everything.
    sta     {1}
    sta     {2}
    sta     {3}
    sta     {3} + 1
    sta     {4}
    rts
    endm

; Sound stop subroutines.  Defined by the macro above.
stop_sound_1 subroutine
    stop_sound AUDC1,AUDF1,sound_pointer_1,sound_timer_1

stop_sound_2 subroutine
    stop_sound AUDC2,AUDF2,sound_pointer_2,sound_timer_2

stop_sound_3 subroutine
    stop_sound AUDC3,AUDF3,sound_pointer_3,sound_timer_3

stop_sound_4 subroutine
    stop_sound AUDC4,AUDF4,sound_pointer_4,sound_timer_4

; Convenience routine for halting all sounds.
stop_all_sounds subroutine
    jsr     stop_sound_1
    jsr     stop_sound_2
    jsr     stop_sound_3
    jmp     stop_sound_4            ; Tail call.

