; This file is part of Magical Fairy Force.

; Magical Fairy Force is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Magical Fairy Force is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Magical Fairy Force.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2020 Average Software.

; Zevasha character data and logic routines

; Zevasha's attack flow:

; Beams will fire from the center of the
; screen.  There will be total of 16 beams,
; and they will be timed so that 4 will be
; active at any given time.

; The beams will generate in 31 frame intervals.
; Each beam will last for 120 frames.  They will
; proceeed through the following phases:

; Warmup - A beam source appears at the center of
;          the screen for 10 frames.  This is a
;          warning to the player.
; Descend - The beam will begin firing, moving two
;           rows every frame.  It will take 33 frames
;           for the beam to reach the bottom of the screen.
; Hold - The beam will remain for 44 frames.
; Clear - The beam will begin to disappear, starting
;         from the top.  It will disappear two rows at a time.
;         This will take 33 frames.

; Beam phase constants.
zevasha_beam_warmup     equ 0
zevasha_beam_descend    equ 1
zevasha_beam_hold       equ 2
zevasha_beam_clear      equ 3

; Phase length constants.
zevasha_warmup_length   equ 11      ; Warmup length is one higher
zevasha_descend_length  equ 33      ; than described to offset the
zevasha_hold_length     equ 44      ; processing on the spawn frame.
zevasha_clear_length    equ 33

; Other attack constants.
zevasha_attack_cost     equ -max_fairy_power * 3 / 4
zevasha_total_beams     equ 16
zevasha_beam_color_1    equ $60
zevasha_beam_color_2    equ $90
zevasha_source_color_0  equ $6c
zevasha_source_color_1  equ $62
zevasha_spawn_ticks     equ 31

; Attack block data offsets.
zevasha_beam_count      equ 0
zevasha_beam_base_low   equ 1
zevasha_beam_base_high  equ 5
zevasha_beam_front_low  equ 9
zevasha_beam_front_high equ 13
zevasha_beam_state      equ 17
zevasha_beam_color      equ 21
zevasha_next_beam_index equ 22
zevasha_spawn_count     equ 23
zevasha_beam_position   equ 24
zevasha_beam_ticks      equ 28

zevasha_start_attack subroutine
    beq     .attack_player_0
    ldx     #zevasha_attack_cost
    jsr     general_player_0_attack_start
    mva     #zevasha_source_color_0,p1_playfield_0_color
    mva     #zevasha_source_color_1,p1_playfield_1_color
    lda     #>p0_attack_data        ; Branch is always true.
    bne     zevasha_init_attack_block
.attack_player_0
    ldx     #zevasha_attack_cost
    jsr     general_player_1_attack_start
    mva     #zevasha_source_color_0,p0_playfield_0_color
    mva     #zevasha_source_color_1,p0_playfield_1_color
    lda     #>p1_attack_data
    ; Fall through.

; Initialize the player attack block
; whose high byte is in the accumulator.
zevasha_init_attack_block subroutine
    sta     $ff
    lda     #0
    sta     $fe
    ldy     #zevasha_beam_count
    lda     #zevasha_total_beams
    sta     ($fe),y
    ldy     #zevasha_spawn_count
    lda     #0                      ; Spawn on first process.
    sta     ($fe),y
    ldy     #zevasha_beam_position + 3
    lda     #-1
.beam_index_loop
    sta     ($fe),y
    cpy     #zevasha_beam_position
    beq     .beam_index_loop_done
    dey
    bpl     .beam_index_loop        ; Always true.
.beam_index_loop_done
    lda     #zevasha_beam_color_1
    ldy     #zevasha_beam_color
    sta     ($fe),y
    ldy     #zevasha_next_beam_index
    lda     #0
    sta     ($fe),y
    ldy     #zevasha_beam_color
    lda     #zevasha_beam_color_1
    sta     ($fe),y
    jmp     start_attack_return

zevasha_process_attack subroutine
    bne     zevasha_attack_player_1
    jmp     zevasha_attack_player_0

zevasha_attack_player_1 subroutine
    lda     #2
    sta     p1_cloud_timer          ; Suppress cloud spawns.
    mwa     p0_attack_data,$fe
    jsr     zevasha_flip_attack_color
    sta     p1_playfield_2_color
    lda     pf_high
    beq     .no_damage
    jsr     damage_player_1
.no_damage
    lda     p0_attack_data + zevasha_beam_count
    beq     .no_spawn               ; All done spawning.
    dec     p0_attack_data + zevasha_spawn_count
    bpl     .no_spawn               ; Not time to spawn yet.
    mwa     zevasha_beam_sound,$fe
    jsr     play_sound_3
    mwa     p0_attack_data,$fe
    jsr     get_player_0_random
    jsr     zevasha_find_spawn_position
    pha                             ; Save the byte offset.
    ldx     #5
.beam_source_loop
    lda     zevasha_beam_source,x
    sta     $2a51,y
    dex
    lda     zevasha_beam_source,x
    sta     $2a50,y
    cpx     #0
    beq     .beam_source_done
    dex
    tya                             ; Add 40 to reach the next row.
    clc
    adc     #40
    tay
    jmp     .beam_source_loop
.beam_source_done
    pla                             ; Retrieve the byte offset.
    ldy     p0_attack_data + zevasha_next_beam_index
    sta     p0_attack_data + zevasha_beam_base_low,y
    sta     p0_attack_data + zevasha_beam_front_low,y
    lda     #$2a                    ; $2a00 + the offset is the beam address.
    sta     p0_attack_data + zevasha_beam_base_high,y
    sta     p0_attack_data + zevasha_beam_front_high,y
    lda     #zevasha_beam_warmup
    sta     p0_attack_data + zevasha_beam_state,y
    lda     #zevasha_warmup_length
    sta     p0_attack_data + zevasha_beam_ticks,y
    dec     p0_attack_data + zevasha_beam_count
    lda     #zevasha_spawn_ticks
    sta     p0_attack_data + zevasha_spawn_count
    cpy     #3
    bne     .no_index_wrap
    ldy     #-1
.no_index_wrap
    iny
    sty     p0_attack_data + zevasha_next_beam_index
.no_spawn
    ldy     #3                      ; Process the beams.
.process_beam_loop
    lda     p0_attack_data + zevasha_beam_position,y
    bpl     .process_beam           ; Negative position is inactive.
    jmp     .process_next_beam
.process_beam
    ldx     p0_attack_data + zevasha_beam_ticks,y
    dex
    txa
    sta     p0_attack_data + zevasha_beam_ticks,y
    lda     p0_attack_data + zevasha_beam_state,y
    bne     .not_warmup
    jmp     .process_warmup
.not_warmup
    cmp     #zevasha_beam_descend
    beq     .process_descend
    cmp     #zevasha_beam_hold
    beq     .process_hold
    txa                             ; Check for tick 0.
    bmi     .clear_finished
    lda     p0_attack_data + zevasha_beam_base_low,y
    sta     $fe
    lda     p0_attack_data + zevasha_beam_base_high,y
    sta     $ff
    sty     $fd                     ; Save the beam index.
    lda     #0
    jsr     zevasha_apply_beam_pattern
    ldy     $fd                     ; Retrieve the index.
    lda     $fe
    sec
    sbc     #80                     ; Move back two rows.
    sta     p0_attack_data + zevasha_beam_base_low,y
    lda     $ff
    sbc     #0
    sta     p0_attack_data + zevasha_beam_base_high,y
    jmp     .process_next_beam
.clear_finished
    lda     p0_attack_data + zevasha_beam_position,y
    pha                             ; Save the position.
    lda     #-1                     ; Disable the beam.
    sta     p0_attack_data + zevasha_beam_position,y
    pla                             ; Get the position back.
    sty     $fd                     ; Save the index.
    asl                             ; Double the position to get the
    tay                             ; offset to the source for erasure.
    lda     #0
    sta     $2a50,y
    sta     $2a51,y
    sta     $2a78,y
    sta     $2a79,y
    sta     $2aa0,y
    sta     $2aa1,y
    ldy     $fd                     ; Retrieve the index.
    jmp     .process_next_beam
.process_hold
    txa                             ; Check for tick 0.
    bpl     .process_next_beam
    lda     #zevasha_beam_clear
    sta     p0_attack_data + zevasha_beam_state,y
    lda     #zevasha_clear_length
    sta     p0_attack_data + zevasha_beam_ticks,y
    bne     .process_next_beam      ; Always true.
.process_descend
    txa                             ; Check for tick 0.
    bmi     .descend_finished
    lda     p0_attack_data + zevasha_beam_front_low,y
    sta     $fe
    lda     p0_attack_data + zevasha_beam_front_high,y
    sta     $ff
    sty     $fd                     ; Save the beam index.
    lda     #$ff
    jsr     zevasha_apply_beam_pattern
    ldy     $fd                     ; Retrieve the index.
    lda     $fe
    sec
    sbc     #80                     ; Move back two rows.
    sta     p0_attack_data + zevasha_beam_front_low,y
    lda     $ff
    sbc     #0
    sta     p0_attack_data + zevasha_beam_front_high,y
    bne     .process_next_beam      ; Always true.
.descend_finished
    lda     #zevasha_beam_hold
    sta     p0_attack_data + zevasha_beam_state,y
    lda     #zevasha_hold_length
    sta     p0_attack_data + zevasha_beam_ticks,y
    bne     .process_next_beam      ; Always true.
.process_warmup
    txa                             ; Check for tick 0.
    bpl     .process_next_beam
    lda     #zevasha_beam_descend
    sta     p0_attack_data + zevasha_beam_state,y
    lda     #zevasha_descend_length
    sta     p0_attack_data + zevasha_beam_ticks,y
.process_next_beam
    dey
    bmi     .check_for_completion
    jmp     .process_beam_loop
.check_for_completion
    lda     p0_attack_data + zevasha_beam_count
    bne     .not_done               ; Still beams to spawn.
    ldy     #3                      ; See if all beams are done.
.beam_done_loop
    lda     p0_attack_data + zevasha_beam_position,y
    bpl     .not_done
    dey
    bpl     .beam_done_loop
    jmp     end_player_0_attack
.not_done
    jmp     p0_process_return

zevasha_attack_player_0 subroutine
    lda     #2                      ; Suppress cloud spawns.
    sta     p0_cloud_timer
    mwa     p1_attack_data,$fe
    jsr     zevasha_flip_attack_color
    sta     p0_playfield_2_color
    lda     pf_low
    beq     .no_damage
    jsr     damage_player_0
.no_damage
    lda     p1_attack_data + zevasha_beam_count
    beq     .no_spawn               ; All done spawning.
    dec     p1_attack_data + zevasha_spawn_count
    bpl     .no_spawn               ; Not time to spawn yet.
    mwa     zevasha_beam_sound,$fe
    jsr     play_sound_4
    mwa     p1_attack_data,$fe
    jsr     get_player_1_random
    jsr     zevasha_find_spawn_position
    pha                             ; Save the byte offset.
    ldx     #0
.beam_source_loop
    lda     zevasha_beam_source,x
    sta     $3140,y
    inx
    lda     zevasha_beam_source,x
    sta     $3141,y
    cpx     #5
    beq     .beam_source_done
    inx
    tya                             ; Add 40 to reach the next row.
    clc
    adc     #40
    tay
    jmp     .beam_source_loop
.beam_source_done
    pla                             ; Retrieve the byte offset.
    ldy     p1_attack_data + zevasha_next_beam_index
    clc
    adc     #$b8                    ; $31b8 + the offset is the beam address.
    sta     p1_attack_data + zevasha_beam_base_low,y
    sta     p1_attack_data + zevasha_beam_front_low,y
    lda     #$31
    sta     p1_attack_data + zevasha_beam_base_high,y
    sta     p1_attack_data + zevasha_beam_front_high,y
    lda     #zevasha_beam_warmup
    sta     p1_attack_data + zevasha_beam_state,y
    lda     #zevasha_warmup_length
    sta     p1_attack_data + zevasha_beam_ticks,y
    dec     p1_attack_data + zevasha_beam_count
    lda     #zevasha_spawn_ticks
    sta     p1_attack_data + zevasha_spawn_count
    cpy     #3
    bne     .no_index_wrap
    ldy     #-1
.no_index_wrap
    iny
    sty     p1_attack_data + zevasha_next_beam_index
.no_spawn
    ldy     #3                      ; Process the beams.
.process_beam_loop
    lda     p1_attack_data + zevasha_beam_position,y
    bpl     .process_beam           ; Negative position is inactive.
    jmp     .process_next_beam
.process_beam
    ldx     p1_attack_data + zevasha_beam_ticks,y
    dex
    txa
    sta     p1_attack_data + zevasha_beam_ticks,y
    lda     p1_attack_data + zevasha_beam_state,y
    bne     .not_warmup
    jmp     .process_warmup
.not_warmup
    cmp     #zevasha_beam_descend
    beq     .process_descend
    cmp     #zevasha_beam_hold
    beq     .process_hold
    txa                             ; Check for tick 0.
    bmi     .clear_finished
    lda     p1_attack_data + zevasha_beam_base_low,y
    sta     $fe
    lda     p1_attack_data + zevasha_beam_base_high,y
    sta     $ff
    sty     $fd                     ; Save the beam index.
    lda     #0
    jsr     zevasha_apply_beam_pattern
    ldy     $fd                     ; Retrieve the index.
    lda     $fe
    clc
    adc     #80                     ; Move ahead two rows.
    sta     p1_attack_data + zevasha_beam_base_low,y
    lda     $ff
    adc     #0
    sta     p1_attack_data + zevasha_beam_base_high,y
    jmp     .process_next_beam
.clear_finished
    lda     p1_attack_data + zevasha_beam_position,y
    pha                             ; Save the position.
    lda     #-1                     ; Disable the beam.
    sta     p1_attack_data + zevasha_beam_position,y
    pla                             ; Get the position back.
    sty     $fd                     ; Save the index.
    asl                             ; Double the position to get the
    tay                             ; offset to the source for erasure.
    lda     #0
    sta     $3140,y
    sta     $3141,y
    sta     $3168,y
    sta     $3169,y
    sta     $3190,y
    sta     $3191,y
    ldy     $fd                     ; Retrieve the index.
    jmp     .process_next_beam
.process_hold
    txa                             ; Check for tick 0.
    bpl     .process_next_beam
    lda     #zevasha_beam_clear
    sta     p1_attack_data + zevasha_beam_state,y
    lda     #zevasha_clear_length
    sta     p1_attack_data + zevasha_beam_ticks,y
    bne     .process_next_beam      ; Always true.
.process_descend
    txa                             ; Check for tick 0.
    bmi     .descend_finished
    lda     p1_attack_data + zevasha_beam_front_low,y
    sta     $fe
    lda     p1_attack_data + zevasha_beam_front_high,y
    sta     $ff
    sty     $fd                     ; Save the beam index.
    lda     #$ff
    jsr     zevasha_apply_beam_pattern
    ldy     $fd                     ; Retrieve the index.
    lda     $fe
    clc
    adc     #80                     ; Move ahead two rows.
    sta     p1_attack_data + zevasha_beam_front_low,y
    lda     $ff
    adc     #0
    sta     p1_attack_data + zevasha_beam_front_high,y
    bne     .process_next_beam      ; Always true.
.descend_finished
    lda     #zevasha_beam_hold
    sta     p1_attack_data + zevasha_beam_state,y
    lda     #zevasha_hold_length
    sta     p1_attack_data + zevasha_beam_ticks,y
    bne     .process_next_beam      ; Always true.
.process_warmup
    txa                             ; Check for tick 0.
    bpl     .process_next_beam
    lda     #zevasha_beam_descend
    sta     p1_attack_data + zevasha_beam_state,y
    lda     #zevasha_descend_length
    sta     p1_attack_data + zevasha_beam_ticks,y
.process_next_beam
    dey
    bmi     .check_for_completion
    jmp     .process_beam_loop
.check_for_completion
    lda     p1_attack_data + zevasha_beam_count
    bne     .not_done               ; Still beams to spawn.
    ldy     #3                      ; See if all beams are done.
.beam_done_loop
    lda     p1_attack_data + zevasha_beam_position,y
    bpl     .not_done
    dey
    bpl     .beam_done_loop
    jmp     end_player_1_attack
.not_done
    jmp     p1_process_return

; Flip the beam color.  Expects the attack block in $fe-$ff.
; The new color will be in the accumulator so it can
; be written into the correct color location.
zevasha_flip_attack_color subroutine
    ldy     #zevasha_beam_color
    lda     ($fe),y
    cmp     #zevasha_beam_color_1
    beq     .flip_to_color_2
    lda     #zevasha_beam_color_1
    .byte   $2c
.flip_to_color_2
    lda     #zevasha_beam_color_2
    sta     ($fe),y
    rts

; Apply the beam pattern in the accumulator to
; two playfields rows starting at $fe-$ff.
zevasha_apply_beam_pattern subroutine
    ldy     #0
    sta     ($fe),y
    iny
    sta     ($fe),y
    ldy     #40                     ; Do the second row.
    sta     ($fe),y
    iny
    sta     ($fe),y
    rts

; Find a spawn position for the next beam.  Expects a
; player's random number in the accumulator and the
; address of the player's data block in $fe-$ff.
; The byte offset to the new position will be in
; the accumulator and the y register on return.
zevasha_find_spawn_position subroutine
    and     #$1f                    ; Trim down to about 20.
    cmp     #20                     ; If the position is 20+, drop
    bmi     .in_bounds              ; it into range.
    sec
    sbc     #20
.in_bounds
    ldy     #zevasha_beam_position + 3
.position_check_loop
    cmp     ($fe),y
    bne     .continue_position_check
    clc
    adc     #1                      ; Try the next position.
    cmp     #20                     ; Wrap to 0.
    bne     .in_bounds              ; Re-check.
    lda     #0
    beq     .in_bounds              ; Always true.
.continue_position_check
    dey
    bpl     .position_check_loop
    pha                             ; Save the position.
    ldy     #zevasha_next_beam_index
    lda     ($fe),y
    clc
    adc     #zevasha_beam_position
    tay
    pla
    sta     ($fe),y
    asl                             ; Double the index to get the offset.
    tay
    rts

zevasha_cpu subroutine
    ldy     #3                      ; Check all four beams.
.beam_loop
    lda     p0_attack_data + zevasha_beam_position,y
    bmi     .next_beam              ; Inactive on -1.
    lda     p0_attack_data + zevasha_beam_base_low,y
    asl                             ; Multiply by 4 to get the
    asl                             ; horizontal pixel offset.
    clc
    adc     #left_bound             ; Get the true horizontal position.
    sec                             ; Check against the CPU's hpos.
    sbc     p1_real_hpos
    bcc     .check_minus
    sbc     #8                      ; Check for danger.
    bcs     .next_beam
    adc     p1_real_hpos
    sta     cpu_xdest
    lda     cpu_left_speed
    sta     cpu_hspeed
    bne     .target_set             ; Always true.
.check_minus
    adc     #8                      ; Check for danger.
    bcc     .next_beam
    clc
    adc     p1_real_hpos
    sta     cpu_xdest
    lda     cpu_right_speed
    sta     cpu_hspeed
.target_set
    inc     cpu_state
    rts
.next_beam
    dey
    bpl     .beam_loop
    lda     p1_real_hpos            ; No danger found.
    sta     cpu_xdest
    lda     p1_vpos
    sta     cpu_ydest
    lda     #0
    sta     cpu_hspeed
    sta     cpu_vspeed
    rts

; Sprite images.
zevasha_p1_sprite
    .byte       %00000000
    .byte       %00000000
    .byte       %10000000
    .byte       %10000000
    .byte       %10000000
    .byte       %00000000
    .byte       %00000100
    .byte       %00000110
    .byte       %00001000
    .byte       %00011100
    .byte       %00111100
    .byte       %00111100
    .byte       %01111100
    .byte       %01111100
    .byte       %01111110
    .byte       %11111110
    .byte       %11111110
    .byte       %11111110
    .byte       %10111111
    .byte       %00100100

zevasha_p2_sprite
    .byte       %10000000
    .byte       %10000000
    .byte       %00010100
    .byte       %00000100
    .byte       %00100000
    .byte       %00000100
    .byte       %00010000
    .byte       %00010001
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000

zevasha_p3_sprite
    .byte       %00000000
    .byte       %00011100
    .byte       %00101000
    .byte       %00111000
    .byte       %01011000
    .byte       %01111000
    .byte       %11101000
    .byte       %10101000
    .byte       %01110000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000

; Letter data for the select screen.
; See pink_b.s for details.
zevasha_letter_data
    .word       big_letter_z,$3000,$29ca
    .byte       big_letter_z_width
    .word       big_letter_e,$3002,$29cc
    .byte       big_letter_e_width
    .word       big_letter_v,$3004,$29ce
    .byte       big_letter_v_width
    .word       big_letter_a,$3006,$29d0
    .byte       big_letter_a_width
    .word       big_letter_s,$3008,$29d2
    .byte       big_letter_s_width
    .word       big_letter_h,$300a,$29d4
    .byte       big_letter_h_width
    .word       big_letter_a,$300c,$29d6
    .byte       big_letter_a_width

zevasha_game_row_offset       equ 22
zevasha_game_portrait_offset  equ 59 - zevasha_game_row_offset

zevasha_matrix_data
    .byte 156,164,156,164   ; Player positions.
    .byte 0,0,0,0           ; Missile positions.
    .byte $64,$64,$5e,$5e   ; Colors.

zevasha_matrix_image equ zevasha_matrix_image_real_start - 16
zevasha_matrix_image_real_start
    .byte %00001111         ; Player 0.
    .byte %00011110
    .byte %00010011
    .byte %00010000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00010000
    .byte %00000000
    .byte %00000000
    .byte %00010000

    .byte %01111000         ; Player 1.
    .byte %01111000
    .byte %01001000
    .byte %00101000
    .byte %01000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %10000000
    .byte %00000000
    .byte %00000000
    .byte %00000000

    .byte %00000000         ; Player 2.
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000110
    .byte %00000111
    .byte %01001101
    .byte %00100011
    .byte %00000000
    .byte %00010101
    .byte %00001111
    .byte %00000111
    .byte %00000010
    .byte %00000011
    .byte %00000001
    .byte %00000000

    .byte %00000000         ; Player 3.
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000010
    .byte %11110000
    .byte %10110000
    .byte %11101000
    .byte %10111000
    .byte %11100000
    .byte %11100000
    .byte %11100000
    .byte %00000000
    .byte %01000000
    .byte %11000000
    .byte %10000000

zevasha_matrix_background
    .byte %00000010,%10101010,%10101010,%10000000
    .byte %00000010,%10101010,%10101010,%10000000
    .byte %00000010,%10101010,%10101010,%10000000
    .byte %00000010,%10101010,%10101010,%10001000
    .byte %00000010,%10101010,%10101010,%10101000
    .byte %00100010,%10101010,%10101010,%10100000
    .byte %00101010,%10101010,%10101010,%10000000
    .byte %00001010,%10101010,%10101010,%10000000
    .byte %00000010,%10101010,%10101010,%10000000
    .byte %00000010,%10101010,%10101010,%00000000
    .byte %00001010,%10101010,%10101010,%00000000
    .byte %00001010,%10101010,%10101010,%10000000
    .byte %00101010,%10101010,%10101010,%10000000
    .byte %10101010,%10101010,%10101010,%10101000
    .byte %10101010,%10101010,%10101010,%10001000
    .byte %10101010,%10101010,%10101010,%00100010

zevasha_beam_source
    .byte %00010101,%01010100
    .byte %01101010,%10101001
    .byte %01101010,%10101001

zevasha_beam_sound
    .byte $ca
    .byte $30,$5
    .byte $34,$5
    .byte $38,$5
    .byte $3c,$5
    .byte $40,$5
    .byte $0

zevasha_portrait
    include "zevasha_portrait.s"

zevasha_portrait_color_1 equ $0 ; 0, 0, 0
zevasha_portrait_color_2 equ $7e ; 247, 198, 255
zevasha_portrait_color_3 equ $8 ; 123, 123, 123
zevasha_portrait_color_4 equ $52 ; 99, 0, 115
zevasha_portrait_color_5 equ $50 ; 57, 0, 74
zevasha_portrait_color_6 equ $36 ; 189, 57, 41
zevasha_portrait_color_11 equ $f0 ; 8, 0, 0
zevasha_portrait_color_13 equ $34 ; 148, 16, 0
zevasha_portrait_color_15 equ $a0 ; 0, 16, 33

zevasha_portrait_table
    .byte 22 + 128
    .byte zevasha_portrait_color_1
    .byte 6 + 128
    .byte zevasha_portrait_color_2
    .byte 1
    .byte zevasha_portrait_color_1
    .byte 3 + 128
    .byte zevasha_portrait_color_2
    .byte 4 + 128
    .byte zevasha_portrait_color_1
    .byte 1
    .byte zevasha_portrait_color_2
    .byte 3 + 128
    .byte zevasha_portrait_color_1
    .byte 6 + 128
    .byte zevasha_portrait_color_2
    .byte 2
    .byte zevasha_portrait_color_1
    .byte zevasha_portrait_color_2
    .byte 8 + 128
    .byte zevasha_portrait_color_1
    .byte 3
    .byte zevasha_portrait_color_4
    .byte zevasha_portrait_color_1
    .byte zevasha_portrait_color_1
    .byte 3 + 128
    .byte zevasha_portrait_color_4
    .byte 12 + 128
    .byte zevasha_portrait_color_2
    .byte 1
    .byte zevasha_portrait_color_4
    .byte 7 + 128
    .byte zevasha_portrait_color_2
    .byte 6 + 128
    .byte zevasha_portrait_color_1
    .byte 1
    .byte zevasha_portrait_color_2
    .byte 3 + 128
    .byte zevasha_portrait_color_1
    .byte 4 + 128
    .byte zevasha_portrait_color_2
    .byte 1
    .byte zevasha_portrait_color_1
    .byte 3 + 128
    .byte zevasha_portrait_color_2
    .byte 6 + 128
    .byte zevasha_portrait_color_1
    .byte 2
    .byte zevasha_portrait_color_2
    .byte zevasha_portrait_color_1
    .byte 5 + 128
    .byte zevasha_portrait_color_2
    .byte 9
    .byte zevasha_portrait_color_6
    .byte zevasha_portrait_color_4
    .byte zevasha_portrait_color_4
    .byte zevasha_portrait_color_1
    .byte zevasha_portrait_color_4
    .byte zevasha_portrait_color_4
    .byte zevasha_portrait_color_1
    .byte zevasha_portrait_color_5
    .byte zevasha_portrait_color_5
    .byte 3 + 128
    .byte zevasha_portrait_color_4
    .byte 4
    .byte zevasha_portrait_color_1
    .byte zevasha_portrait_color_11
    .byte zevasha_portrait_color_4
    .byte zevasha_portrait_color_1
    .byte 5 + 128
    .byte zevasha_portrait_color_4
    .byte 2
    .byte zevasha_portrait_color_2
    .byte zevasha_portrait_color_1
    .byte 4 + 128
    .byte zevasha_portrait_color_4
    .byte 9 + 128
    .byte zevasha_portrait_color_3
    .byte 6 + 128
    .byte zevasha_portrait_color_13
    .byte 6
    .byte zevasha_portrait_color_1
    .byte zevasha_portrait_color_1
    .byte zevasha_portrait_color_4
    .byte zevasha_portrait_color_11
    .byte zevasha_portrait_color_1
    .byte zevasha_portrait_color_15
    .byte 7 + 128
    .byte zevasha_portrait_color_4
    .byte 5 + 128
    .byte zevasha_portrait_color_6
    .byte 7 + 128
    .byte zevasha_portrait_color_1

